/*
 *
 * Copyright (C) 2009 Red Hat, Inc.
 * Copyright (C) 2013 Collabora Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * Written by: Arun Raghavan
 *
 */

#include "config.h"

#include <stdlib.h>
#include <pthread.h>

#include "ply-gst.h"
#include "ply-logger.h"
#include "ply-boot-splash.h"

ply_boot_splash_plugin_interface_t *ply_boot_splash_plugin_get_interface (void);

struct _ply_boot_splash_plugin
{
  ply_gst_t *g;
};

static ply_boot_splash_plugin_t *
create_plugin (ply_key_file_t *key_file)
{
  ply_boot_splash_plugin_t *plugin;
  ply_gst_t *g = NULL;
  char *pipeline_str = NULL;

  ply_trace ("creating pipeline");
  pipeline_str = ply_key_file_get_value (key_file, "gstreamer", "Pipeline");

  g = ply_gst_new (pipeline_str);
  free (pipeline_str);

  if (g != NULL)
    {
      plugin = calloc (1, sizeof (ply_boot_splash_plugin_t));
      plugin->g = g;
    }
  return plugin;
}

static void
destroy_plugin (ply_boot_splash_plugin_t *plugin)
{
  ply_gst_free (plugin->g);
  free (plugin);
}

static bool
show_splash_screen (ply_boot_splash_plugin_t *plugin,
                    ply_event_loop_t         *loop,
                    ply_buffer_t             *boot_buffer,
                    ply_boot_splash_mode_t    mode)
{
  ply_gst_start (plugin->g);

  return true;
}

static void
update_status (ply_boot_splash_plugin_t *plugin,
               const char               *status)
{
  ply_trace ("Status: %s", status);
  /* FIXME: Maybe add a textoverlay to deal with this */
}

static void
hide_splash_screen (ply_boot_splash_plugin_t *plugin,
                    ply_event_loop_t         *loop)
{
  ply_gst_stop (plugin->g);
}

ply_boot_splash_plugin_interface_t *
ply_boot_splash_plugin_get_interface (void)
{
  static ply_boot_splash_plugin_interface_t plugin_interface =
    {
      .create_plugin = create_plugin,
      .destroy_plugin = destroy_plugin,
      .show_splash_screen = show_splash_screen,
      .update_status = update_status,
      .hide_splash_screen = hide_splash_screen,
    };

  return &plugin_interface;
}
