/* script-lib-gst.c - scripting system ply-gst wrapper
 *
 * Copyright (C) 2009 Charlie Brej <cbrej@cs.man.ac.uk>
 * Copyright (C) 2013 Sjoerd Simons <sjoerd.simons@collabora.co.uk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * Written by: Charlie Brej <cbrej@cs.man.ac.uk>
 */

#include "config.h"

#include "ply-gst.h"
#include "ply-logger.h"
#include "script.h"
#include "script-parse.h"
#include "script-object.h"
#include "script-parse.h"
#include "script-execute.h"
#include "script-lib-gst.h"
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "script-lib-gst.script.h"

static void gst_free (script_obj_t *obj)
{
  ply_gst_t *g = obj->data.native.object_data;

  ply_gst_free (g);
}

static script_return_t gst_new (script_state_t *state,
                                  void           *user_data)
{
  script_obj_t *reply;
  script_lib_gst_data_t *data = user_data;
  char *pipeline = script_obj_hash_get_string (state->local, "pipeline");
  ply_gst_t * g = ply_gst_new (pipeline);

  reply = script_obj_new_native (g, data->class);

  return script_return_obj (reply);
}

static script_return_t gst_start (script_state_t *state, void *user_data)
{
  script_lib_gst_data_t *data = user_data;
  ply_gst_t *g = script_obj_as_native_of_class (state->this, data->class);

  ply_gst_start (g);

  return script_return_obj_null ();
}

static script_return_t gst_stop (script_state_t *state, void *user_data)
{
  script_lib_gst_data_t *data = user_data;
  ply_gst_t *g = script_obj_as_native_of_class (state->this, data->class);

  ply_gst_stop (g);

  return script_return_obj_null ();
}

script_lib_gst_data_t *script_lib_gst_setup (script_state_t *state)
{
  script_lib_gst_data_t *data = malloc (sizeof (script_lib_gst_data_t));

  data->class = script_obj_native_class_new (gst_free, "gst", data);
  script_obj_t *gst_hash = script_obj_hash_get_element (state->global, "Gst");
  
  script_add_native_function (gst_hash,
                              "_New",
                              gst_new,
                              data,
                              "pipeline",
                              NULL);
  script_add_native_function (gst_hash,
                              "Start",
                              gst_start,
                              data,
                              NULL);
  script_add_native_function (gst_hash,
                              "Stop",
                              gst_stop,
                              data,
                              NULL);
  script_obj_unref (gst_hash);

  data->script_main_op = script_parse_string (script_lib_gst_string,
    "script-lib-gst.script");
  script_return_t ret = script_execute (state, data->script_main_op);
  script_obj_unref (ret.object);

  return data;
}

void script_lib_gst_destroy (script_lib_gst_data_t *data)
{
  script_obj_native_class_destroy (data->class);
  script_parse_op_free (data->script_main_op);
  free (data);
}

